package com.compsych.web.context;

import java.util.Iterator;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.application.NavigationHandler;
import javax.faces.application.ViewExpiredException;
import javax.faces.context.ExceptionHandler;
import javax.faces.context.ExceptionHandlerWrapper;
import javax.faces.context.FacesContext;
import javax.faces.event.ExceptionQueuedEvent;
import javax.faces.event.ExceptionQueuedEventContext;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.compsych.web.util.MessageUtils;

public class GlobalExceptionHandler extends ExceptionHandlerWrapper {

    private static Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    public static final String HANDLED_EXCEPTION = "cpp.exception";
    public static final String SHOW_REFRESH = "showRefresh";

    private ExceptionHandler wrapped;

    public GlobalExceptionHandler(ExceptionHandler wrapped) {

        this.wrapped = wrapped;
    }

    @Override
    public ExceptionHandler getWrapped() {

        return this.wrapped;
    }

    @Override
    public void handle() throws FacesException {

        for(Iterator<ExceptionQueuedEvent> i = getUnhandledExceptionQueuedEvents().iterator(); i.hasNext();) {
            ExceptionQueuedEvent event = i.next();
            ExceptionQueuedEventContext context = (ExceptionQueuedEventContext)event.getSource();
            Throwable t = context.getException();
            if(log.isDebugEnabled()) {
                log.debug("An error occurred: " + ExceptionUtils.getRootCauseMessage(t));
            }
            FacesContext fc = FacesContext.getCurrentInstance();
            NavigationHandler nav = fc.getApplication().getNavigationHandler();
            if(t instanceof ViewExpiredException) {
                try {
                    // Push some useful stuff to the request scope for
                    // use in the page
                    fc.addMessage(
                        null,
                        new FacesMessage(FacesMessage.SEVERITY_INFO, MessageUtils
                            .getMessage("session.expired"), null));

                    nav.handleNavigation(fc, null, "/s/view");
                    fc.renderResponse();

                }
                finally {
                    i.remove();
                }
            }
            else {
                // Send other exceptions to a default error page
                Map<String, Object> requestMap = fc.getExternalContext().getRequestMap();
                try {
                    // Push the Exception to the request scope for use in the page
                    requestMap.put(HANDLED_EXCEPTION, t);

                    nav.handleNavigation(fc, null, "/error");
                    fc.renderResponse();

                }
                finally {
                    i.remove();
                }
            }
        }
        // At this point, the queue will not contain any Exceptions we care to handle.
        // Therefore, let the parent handle them.
        getWrapped().handle();

    }
}
